@@ -59,7 +59,8 @@ linc_connection_dispose (GObject *obj)
 		parent_class->dispose (obj);
 }
 
-#define ERR_CONDS (G_IO_ERR|G_IO_HUP|G_IO_NVAL|G_IO_PRI)
+#define ERR_CONDS (G_IO_ERR|G_IO_HUP|G_IO_NVAL)
+#define IN_CONDS  (G_IO_PRI|G_IO_IN)
 
 static gboolean
 linc_connection_connected (GIOChannel  *gioc,
@@ -78,7 +79,7 @@ linc_connection_connected (GIOChannel  *gioc,
 	klass = LINC_CONNECTION_CLASS (G_OBJECT_GET_CLASS (data));
 
 	if (cnx->status == LINC_CONNECTED &&
-	    condition & G_IO_IN &&
+	    condition & IN_CONDS &&
 	    klass->handle_input)
 		retval = klass->handle_input (cnx);
 
@@ -116,9 +117,24 @@ linc_connection_connected (GIOChannel  *gioc,
 	return retval;
 }
 
+/*
+ * linc_connection_class_state_changed:
+ * @cnx: a #LINCConnection
+ * @status: a #LINCConnectionStatus value.
+ *
+ * Set up linc's #GSources if the connection is in the #LINC_CONNECTED
+ * or #LINC_CONNECTING state.
+ *
+ * Remove the #GSources if the state has channged to #LINC_DISCONNECTED,
+ * close the socket and a gobject broken signal which may be caught by
+ * the application.
+ *
+ * Also perform SSL specific operations if the connection has move into
+ * the #LINC_CONNECTED state.
+ */
 static void
-linc_connection_real_state_changed (LINCConnection      *cnx,
-				    LINCConnectionStatus status)
+linc_connection_class_state_changed (LINCConnection      *cnx,
+				     LINCConnectionStatus status)
 {
 	switch (status) {
 	case LINC_CONNECTED:
@@ -132,7 +148,7 @@ linc_connection_real_state_changed (LINCConnection      *cnx,
 #endif
 		if (!cnx->tag)
 			cnx->tag = linc_io_add_watch (
-				cnx->gioc, ERR_CONDS | G_IO_IN,
+				cnx->gioc, ERR_CONDS | IN_CONDS,
 				linc_connection_connected, cnx);
 		break;
 
@@ -144,8 +160,6 @@ linc_connection_real_state_changed (LINCConnection      *cnx,
 		break;
 
 	case LINC_DISCONNECTED:
-/*      g_warning ("Linc disconnected tag %d fd '%d'",
-	cnx->tag, cnx->fd);*/
 		linc_source_remove (cnx, TRUE);
 		linc_close_fd (cnx);
 
@@ -157,93 +171,135 @@ linc_connection_real_state_changed (LINCConnection      *cnx,
 	cnx->status = status;
 }
 
+/*
+ * linc_connection_from_fd:
+ * @cnx: a #LINCConnection.
+ * @fd: a connected/connecting file descriptor.
+ * @proto: a #LINCProtocolInfo.
+ * @host: protocol dependant host information.
+ * @service: protocol dependant service information(e.g. port number).
+ * @was_initiated: #TRUE if the connection was initiated by us.
+ * @status: a #LINCConnectionStatus value.
+ * @options: combination of #LINCConnectionOptions.
+ *
+ * Fill in @cnx, call protocol specific initialisation methonds and then
+ * call linc_connection_state_changed.
+ *
+ * Return Value: #TRUE if the function succeeds, #FALSE otherwise.
+ */
 gboolean
-linc_connection_from_fd (LINCConnection *cnx, int fd,
+linc_connection_from_fd (LINCConnection         *cnx,
+			 int                     fd,
 			 const LINCProtocolInfo *proto,
-			 const char *remote_host_info,
-			 const char *remote_serv_info,
-			 gboolean was_initiated,
-			 LINCConnectionStatus status,
-			 LINCConnectionOptions options)
+			 char                   *host,
+			 char                   *service,
+			 gboolean                was_initiated,
+			 LINCConnectionStatus    status,
+			 LINCConnectionOptions   options)
 {
-  cnx->fd = fd;
-  cnx->gioc = g_io_channel_unix_new (fd);
-  cnx->was_initiated = was_initiated;
-  cnx->is_auth = (proto->flags & LINC_PROTOCOL_SECURE)?TRUE:FALSE;
-  cnx->remote_host_info = g_strdup(remote_host_info);
-  cnx->remote_serv_info = g_strdup(remote_serv_info);
-  cnx->options = options;
+	cnx->fd               = fd;
+	cnx->gioc             = g_io_channel_unix_new (fd);
+	cnx->was_initiated    = was_initiated;
+	cnx->is_auth          = (proto->flags & LINC_PROTOCOL_SECURE) ? TRUE : FALSE;
+	cnx->remote_host_info = host;
+	cnx->remote_serv_info = service;
+	cnx->options          = options;
 
-  if(proto->setup)
-    proto->setup (fd, options);
+	if (proto->setup)
+		proto->setup (fd, options);
 
 #ifdef LINC_SSL_SUPPORT
-  if (options & LINC_CONNECTION_SSL) {
-      cnx->ssl = SSL_new (linc_ssl_ctx);
-      SSL_set_fd (cnx->ssl, fd);
-  }
+	if (options & LINC_CONNECTION_SSL) {
+		cnx->ssl = SSL_new (linc_ssl_ctx);
+		SSL_set_fd (cnx->ssl, fd);
+	}
 #endif
 
-  linc_connection_state_changed (cnx, status);
+	linc_connection_state_changed (cnx, status);
 
-  return TRUE;
+  	return TRUE;
 }
 
-/* This will just create the fd, do the connect and all, and then call
-   linc_connection_from_fd */
+/*
+ * linc_connection_initiate:
+ * @cnx: a #LINCConnection.
+ * @proto_name: the name of the protocol to use.
+ * @host: protocol dependant host information.
+ * @service: protocol dependant service information(e.g. port number).
+ * @options: combination of #LINCConnectionOptions.
+ *
+ * Initiate a connection to @service on @host using the @proto_name protocol.
+ *
+ * Note: this function may be successful without actually having connected
+ *       to @host - the connection handshake may not have completed.
+ *
+ * Return Value: #TRUE if the function succeeds, #FALSE otherwise.
+ */
 gboolean
-linc_connection_initiate (LINCConnection *cnx, const char *proto_name,
-			  const char *remote_host_info,
-			  const char *remote_serv_info,
-			  LINCConnectionOptions options)
+linc_connection_initiate (LINCConnection        *cnx,
+			  const char            *proto_name,
+			  const char            *host,
+			  const char            *service,
+			  LINCConnectionOptions  options)
 {
-  struct addrinfo *ai, hints = {AI_CANONNAME, 0, SOCK_STREAM, 0, 0,
-				NULL, NULL, NULL};
-  int rv;
-  int fd;
-  const LINCProtocolInfo *proto;
-  gboolean retval = FALSE;
+	const LINCProtocolInfo *proto;
+	int                     rv;
+	int                     fd;
+	gboolean                retval = FALSE;
+	struct sockaddr        *saddr;
+	socklen_t		saddr_len;
 
-  proto = linc_protocol_find(proto_name);
+	proto = linc_protocol_find (proto_name);
 
-  if(!proto)
-    return FALSE;
+	if (!proto)
+		return FALSE;
 
-  hints.ai_family = proto->family;
-  rv = linc_getaddrinfo(remote_host_info, remote_serv_info, &hints, &ai);
-  if(rv)
-    return FALSE;
 
-  fd = socket(proto->family, SOCK_STREAM, proto->stream_proto_num);
-/* FIXME: it seems slightly whacked out that socket returns fd == 0 to me. */
-/*  g_warning ("Socket: %d, '%s'", fd, proto_name); */
-  if(fd < 0)
-    goto out;
-  if(options & LINC_CONNECTION_NONBLOCKING) {
-    if (fcntl(fd, F_SETFL, O_NONBLOCK) < 0)
-      goto out;
-  }
-  if (fcntl(fd, F_SETFD, FD_CLOEXEC) < 0)
-    goto out;
+	saddr = linc_protocol_get_sockaddr (proto, host, 
+					    service, &saddr_len);
+	if (!saddr)
+		return FALSE;
 
-  rv = connect(fd, ai->ai_addr, ai->ai_addrlen);
-  if(rv && errno != EINPROGRESS)
-    goto out;
+	fd = socket (proto->family, SOCK_STREAM, 
+		     proto->stream_proto_num);
+
+	if (fd < 0)
+		goto out;
+
+	if (options & LINC_CONNECTION_NONBLOCKING)
+		if (fcntl (fd, F_SETFL, O_NONBLOCK) < 0)
+			goto out;
+
+	if (fcntl (fd, F_SETFD, FD_CLOEXEC) < 0)
+		goto out;
+
+	rv = connect (fd, saddr, saddr_len);
+	if (rv && errno != EINPROGRESS)
+		goto out;
+
+	retval = linc_connection_from_fd (
+				cnx, fd, proto, 
+				g_strdup (host),
+				g_strdup (service),
+				TRUE, rv ? LINC_CONNECTING : LINC_CONNECTED,
+				options);
 
-  retval = linc_connection_from_fd (
-	  cnx, fd, proto, remote_host_info,
-	  remote_serv_info, TRUE, rv?LINC_CONNECTING:LINC_CONNECTED, options);
  out:
-  if(!cnx)
-    {
-      if(fd >= 0)
-	close(fd);
-    }
-  freeaddrinfo(ai);
+	if (!cnx && fd >= 0)
+		close (fd);
 
-  return retval;
+	g_free (saddr);
+
+	return retval;
 }
 
+/*
+ * linc_connection_state_changed:
+ * @cnx: a #LINCConnection.
+ * @status: a #LINCConnectionStatus.
+ *
+ * A wrapper for the #LINCConnectionClass's state change method.
+ */
 void
 linc_connection_state_changed (LINCConnection      *cnx,
 			       LINCConnectionStatus status)
@@ -265,6 +321,9 @@ linc_connection_read (LINCConnection *cnx, guchar *buf,
   int n;
   int written = 0;
 
+  if (!len)
+    return 0;
+
   if(cnx->options & LINC_CONNECTION_NONBLOCKING)
     {
       while(cnx->status == LINC_CONNECTING)
@@ -307,7 +366,7 @@ linc_connection_read (LINCConnection *cnx, guchar *buf,
 	    }
 	}
       else if(n == 0)
-	return -1;
+	return 0;
       else
 	{
 	  buf += n;
@@ -453,7 +512,7 @@ linc_connection_class_init (LINCConnectionClass *klass)
 
 	object_class->dispose = linc_connection_dispose;
 
-	klass->state_changed  = linc_connection_real_state_changed;
+	klass->state_changed  = linc_connection_class_state_changed;
 	klass->broken         = NULL;
 
 	linc_connection_signals [BROKEN] =
